//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:53:09 03/19/2008 
// Design Name: 
// Module Name:    sum_control 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module sum_control(clk, reset, go, en_reg, shift, en_acc, reset_sum, done);

	input clk;
	input go;
	input reset;
	output en_reg;
	output shift;
	output en_acc;
	output reset_sum;
	output done;
	
	//the number of times the accumulator will be employed
	parameter N = 6'd32, INCR = 6'd1;
	  
	//the output of the counter
	reg [5:0] counter;
	 
	//the states that will be used: 4 states
	reg [2:0] current_state, next_state;
	
	//parameters to store values for states
	parameter st_reset = 3'b000, st_load = 3'b001, st_add = 3'b010, st_shift = 3'b011, st_done = 3'b100; 

	
	always @( * )
	begin
		case(current_state)
			st_reset:	
			begin
				next_state = st_load;
			end
			
			st_load:
			begin
				if(go) next_state = st_add;
				else next_state = st_load;
			end
			
			st_add:	   
			begin
				if(counter == N) next_state = st_done;
				else next_state = st_shift;
			end
			
			st_shift:
			begin	
				next_state = st_add;	
			end
			
			st_done:
			begin	
				next_state = st_load;
			end
			
			default: 
			begin
				next_state = st_reset;
			end
		endcase
			
	end

	always @(posedge clk)
	begin
		if(reset) 
			current_state = st_reset;
		else
		begin
			if( next_state == st_load ) counter = 6'd0;
			if( next_state == st_add )
				counter = counter + INCR;
			current_state = next_state;
		end	 
	 end
	
	
	assign en_reg = ((current_state == st_load) | (current_state == st_shift)) ? 1'b1 : 1'b0;
	assign shift = (current_state == st_shift) ? 1'b1 : 1'b0;
	assign en_acc = (current_state == st_add) ? 1'b1 : 1'b0;
	assign reset_sum = (current_state == st_reset) ? 1'b1 : 1'b0;
	assign done = (current_state == st_done) ? 1'b1 : 1'b0;
	

endmodule
