//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:45:57 03/01/2008 
// Design Name: 
// Module Name:    full_adder_16_core 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module full_adder_16_core(Clk,Reset,In_Data_FIFO,In_Data_Exists,In_FIFO_Read,Out_Data_FIFO,Out_FIFO_Full,Out_FIFO_Write);
	parameter C_DWIDTH = 6'd32;
	input Clk;
	input Reset;
	
	/*Slave Signals*/
	input [C_DWIDTH-1:0] In_Data_FIFO;
	input In_Data_Exists;
	output In_FIFO_Read;
	
	/*Master Signals*/
	output [C_DWIDTH-1:0] Out_Data_FIFO;
	input Out_FIFO_Full;
	output Out_FIFO_Write;
	
	
	//the states that will be used: 4 states
	reg [2:0] current_state, next_state;
	
	wire [C_DWIDTH-1:0] t_in;
	wire [15:0] t_out;
	wire gnd,cout;
	wire write;

	//parameters to store values for states
	parameter st_reset = 3'b000, st_read = 3'b001, st_add = 3'b010, st_write = 3'b011, st_done = 3'b100; 
	assign gnd = 1'b0;
	
	always @( * )
	begin
		case(current_state)
			st_reset:	
			begin
				if(!In_Data_Exists) next_state = st_reset;
				else 
				begin
					next_state = st_read; 
				end
			end
			
			st_read:
			begin
				next_state = st_add;
			end
			
			st_add:	   
			begin
				if(!Out_FIFO_Full) next_state = st_write;
				else next_state = st_add;
			end
			
			st_write:
			begin	
				next_state = st_done;
			end
			
			st_done:
			begin	
				next_state = st_reset;
			end
			default: next_state = st_reset;
		endcase
			
	end

	always @(posedge Clk)
	begin
		if(Reset) current_state <= st_reset;
	else current_state <= next_state;
	end
	 
	
	 
	assign In_FIFO_Read = (current_state == st_read) ? 1'b1 : 1'b0;
	assign Out_FIFO_Write = (current_state == st_done) ? 1'b1 : 1'b0;
	assign write = (current_state == st_write) ? 1'b1 : 1'b0;
	assign Out_Data_FIFO[31:16] = 16'd0;
	
	nn_register_16 reg1(Clk,In_Data_FIFO[15:0],Reset,In_FIFO_Read,t_in[15:0]);
	nn_register_16 reg2(Clk,In_Data_FIFO[31:16],Reset,In_FIFO_Read,t_in[31:16]);
	full_adder16 adder(t_in[31:16],t_in[15:0],gnd,t_out,cout);
	nn_register_16 reg3(Clk,t_out,Reset,write,Out_Data_FIFO[15:0]);
	
	

endmodule
