//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    18:28:01 02/26/2008 
// Design Name: 
// Module Name:    nn_control 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module nn_control(clk, reset, go, train_test, en, rs, done);
	input clk;
	input go;
	input reset;
	input train_test;
	output [2:0] en;
	output [2:0] rs;
	output done;
	
	//the number of times the accumulator will be employed
	parameter N = 7'd100, INCR = 7'd1;
	  
	//the output of the counter
	reg [7:0] counter;
	 
	//the states that will be used: 4 states
	reg [2:0] current_state, next_state;
	reg [2:0] en_out;
	reg [2:0] rs_out;

	//parameters to store values for states
	parameter st_reset = 3'b00, st_idle = 3'b001, st_add = 3'b010, st_load = 3'b011, st_done = 3'b100; 

	
	always @( * )
	begin
		case(current_state)
			st_reset:	
			begin
				en_out = 3'b000;
				rs_out = 3'b111;
				next_state = st_idle; 
			end
			
			st_idle:
			begin
				en_out = 3'b000;
				rs_out = 3'b110;		
				if(!go) next_state = st_idle;
				else next_state = st_add;
			end
			
			st_add:	   
			begin
				en_out = 3'b110;
				rs_out = 3'b000;
				if(counter==N) 
				begin	
					if(train_test) next_state = st_done;
					else next_state = st_load;
				end
				else next_state = st_add;
			end
			
			st_load:
			begin	
				en_out = 3'b001;
				rs_out = 3'b000;
				next_state = st_done; 
			end
			
			st_done:
			begin	
				en_out = 3'b000;
				rs_out = 3'b000;
				next_state = st_idle; 
			end
			
			default: 
			begin
				en_out = 3'b000;
				rs_out = 3'b111;
				next_state = st_reset;
			end
		endcase
			
	end

	always @(posedge clk)
	begin
		if(reset)
		begin
			current_state = st_reset;
			counter = 7'd0;
		end
		else
		begin
			if(next_state == st_idle) counter = 7'd0;
			if( next_state == st_add )
				counter = counter + INCR;
			current_state = next_state;
		end
	 end

	assign en = en_out; 
   assign rs = rs_out;     
	assign done = (current_state == st_done) ? 1'b1 : 1'b0;
	 
		
endmodule
