//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:45:57 03/01/2008 
// Design Name: 
// Module Name:    full_adder_16_core 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module nn_interface(Clk,Reset,In_Data_FIFO,In_Data_Exists,In_FIFO_Read,Out_Data_FIFO,Out_FIFO_Full,Out_FIFO_Write);
	parameter C_DWIDTH = 6'd32;
	input Clk;
	input Reset;
	
	/*Slave Signals*/
	input [C_DWIDTH-1:0] In_Data_FIFO;
	input In_Data_Exists;
	output In_FIFO_Read;
	
	/*Master Signals*/
	output [C_DWIDTH-1:0] Out_Data_FIFO;
	input Out_FIFO_Full;
	output Out_FIFO_Write;
	
	
	//the states that will be used: 4 states
	reg [2:0] current_state, next_state;
	
	//counter
	reg [2:0] counter;
	
	wire read_1, read_2, read_3, read_4;
	wire write;
	wire go;
	
	wire [98:0] in_vector;
	
	
	wire done;
	wire [2:0] cont_reset;
	wire [2:0] cont_enable;
	
	wire [9:0] neural_result;
	
	wire [31:0] out_reg1;
	wire [31:0] out_reg2;
	wire [31:0] out_reg3;
	wire [31:0] out_reg4;
	
	

	//parameters to store values for states
	parameter st_reset = 3'b000, st_read = 3'b001, st_delay = 3'b010, st_write = 3'b011, st_go = 3'b100, st_idle = 3'b101, st_done = 3'b110; 

	
	always @( * )
	begin
		case(current_state)
			
			st_reset:	
			begin
				if(counter == 3'b100)
				   next_state = st_delay;
				else
				begin
					if(!In_Data_Exists) next_state = st_reset;
					else next_state = st_read; 
				end
			end
			
			st_read:
			begin
				next_state = st_reset;
			end
			
			st_delay:
			begin
				next_state = st_write;
			end
			
			st_write:
			begin	
				next_state = st_go;
			end
			
			st_go:
			begin
				next_state = st_idle;
			end
			
			st_idle:
			begin
				if(!Out_FIFO_Full & done)
					next_state = st_done;
				else next_state = st_idle;
			end
			
			st_done:
			begin	
				next_state = st_reset;
			end
			
			default: next_state = st_reset;
		endcase
			
	end

	always @(posedge Clk)
	begin
		if(~Reset) 
		begin
			current_state <= st_reset;
			counter = 3'b000;
		end
		else
		begin
			if(next_state == st_read)
				counter = counter + 2'b01;
			if(next_state == st_done)
				counter = 2'b00;
			current_state <= next_state;
		end
	end
	 

	
	assign read_1 = ((current_state == st_read) & (counter == 3'b001)) ? 1'b1: 1'b0;
	assign read_2 = ((current_state == st_read) & (counter == 3'b010)) ? 1'b1: 1'b0;
	assign read_3 = ((current_state == st_read) & (counter == 3'b011)) ? 1'b1: 1'b0;
	assign read_4 = ((current_state == st_read) & (counter == 3'b100)) ? 1'b1: 1'b0;
	
	assign In_FIFO_Read = ((current_state == st_read)) ? 1'b1 : 1'b0;
	
	/*assign Out_FIFO_Write = (current_state == st_done) ? 1'b1 : 1'b0;
	assign Out_Data_FIFO = {out_reg1[31:24],out_reg2[23:16],out_reg3[15:8],out_reg4[7:0]};
	assign write = (current_state == st_write) ? 1'b1 : 1'b0;
	assign go = (current_state == st_delay) ? 1'b1 : 1'b0;*/
	
	assign write = (current_state == st_write) ? 1'b1 : 1'b0;
	assign go = (current_state == st_go) ? 1'b1 : 1'b0;
	/*assign Out_FIFO_Write = ((current_state==st_write) | (current_state==st_done))?1'b1 :1'b0;*/
	assign Out_FIFO_Write = (write | done);
	
	assign in_vector[98:96] = out_reg1[2:0];
	assign in_vector[95:64] = out_reg2[31:0];
	assign in_vector[63:32] = out_reg3[31:0];
	assign in_vector[31:0] = out_reg4[31:0];

	
	nn_register_32 reg1(Clk,In_Data_FIFO,~Reset,read_1,out_reg1);
	nn_register_32 reg2(Clk,In_Data_FIFO,~Reset,read_2,out_reg2);
	nn_register_32 reg3(Clk,In_Data_FIFO,~Reset,read_3,out_reg3);
	nn_register_32 reg4(Clk,In_Data_FIFO,~Reset,read_4,out_reg4);
	
	nn_testing neural1(Clk, out_reg1[30], cont_enable, cont_reset, in_vector, neural_result[0]);
	nn_testing neural2(Clk, out_reg1[29], cont_enable, cont_reset, in_vector, neural_result[1]);
	nn_testing neural3(Clk, out_reg1[28], cont_enable, cont_reset, in_vector, neural_result[2]);
	nn_testing neural4(Clk, out_reg1[27], cont_enable, cont_reset, in_vector, neural_result[3]);
	nn_testing neural5(Clk, out_reg1[26], cont_enable, cont_reset, in_vector, neural_result[4]);
	
	nn_testing neural6(Clk, out_reg1[25], cont_enable, cont_reset, in_vector, neural_result[5]);
	nn_testing neural7(Clk, out_reg1[24], cont_enable, cont_reset, in_vector, neural_result[6]);
	nn_testing neural8(Clk, out_reg1[23], cont_enable, cont_reset, in_vector, neural_result[7]);
	nn_testing neural9(Clk, out_reg1[22], cont_enable, cont_reset, in_vector, neural_result[8]);
	nn_testing neural10(Clk, out_reg1[21], cont_enable, cont_reset, in_vector, neural_result[9]);
	
	nn_control control(Clk, ~Reset, go, out_reg1[31], cont_enable, cont_reset, done);
	
	assign Out_Data_FIFO[9:0] = neural_result[9:0];
	assign Out_Data_FIFO[31:10] = 27'b0000000000000000000000;
	
	
	
endmodule
