//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:45:57 03/01/2008 
// Design Name: 
// Module Name:    full_adder_16_core 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module ru_interface(Clk,Reset,In_Data_FIFO,In_Data_Exists,In_FIFO_Read,Out_Data_FIFO,Out_FIFO_Full,Out_FIFO_Write);
	parameter C_DWIDTH = 6'd32;
	input Clk;
	input Reset;
	
	/*Slave Signals*/
	input [C_DWIDTH-1:0] In_Data_FIFO;
	input In_Data_Exists;
	output In_FIFO_Read;
	
	/*Master Signals*/
	output [C_DWIDTH-1:0] Out_Data_FIFO;
	input Out_FIFO_Full;
	output Out_FIFO_Write;
	
	
	//the states that will be used: 4 states
	reg [2:0] current_state, next_state;
	
	//counters
	reg [3:0] counter_9;
	reg [5:0] counter_32;
	
	//some control signals
	wire internal_done;
	wire internal_reset;
	wire [8:0] out;
	wire go;
	wire done;
	
	//enable signsl to store the data txed over the FSL
	wire read_1, read_2, read_3, read_4, read_5, read_6, read_7, read_8, read_9;
	
	//registers to store the data
	wire [31:0] out_reg1;
	wire [31:0] out_reg2;
	wire [31:0] out_reg3;
	wire [31:0] out_reg4;
	wire [31:0] out_reg5;
	wire [31:0] out_reg6;
	wire [31:0] out_reg7;
	wire [31:0] out_reg8;
	wire [31:0] out_reg9;
	
	

	//parameters to store values for states
	parameter st_reset = 3'b000, st_row_read = 3'b001, st_col_read = 3'b010, st_col_wait = 3'b011, st_go = 3'b100, st_idle = 3'b101, 
				 st_row_wait = 3'b111, st_done = 3'b110; 

	
	always @( * )
	begin
		case(current_state)
			st_reset:
			begin
			if(!In_Data_Exists) next_state = st_reset;
			else next_state = st_row_read;
			end
			st_row_read:	
			begin
				next_state = st_col_read; 
			end
			
			st_col_read:
			begin
				 next_state = st_col_wait; 
			end
			
			st_col_wait:
			begin
				if(counter_9 == 4'b1001)
						next_state = st_go;
				else
				begin
					if(!In_Data_Exists) next_state = st_col_wait;
					else next_state = st_col_read; 
				end
			end
			
			st_go:
			begin
				next_state = st_idle;
			end
			
			st_idle:
			begin
				if(!internal_done) next_state = st_idle;
				else next_state = st_row_wait;
			end
			
			st_row_wait:
			begin
				if(counter_32 == 6'b100000)
						next_state = st_done;
				else
				begin
					if(!In_Data_Exists) next_state = st_row_wait;
					else next_state = st_row_read; 
				end
			end
			
			st_done:
			begin	
				next_state = st_reset;
			end
			
			default: next_state = st_reset;
		endcase
			
	end

	always @(posedge Clk)
	begin
		if(~Reset) 
		begin
			current_state <= st_reset;
			counter_9 = 4'b0000;
			counter_32 = 6'b000000;
		end
		else
		begin
			if(next_state == st_col_read)
				counter_9 = counter_9 + 4'b0001;
			if(next_state == st_row_read)
				counter_32 = counter_32 + 6'b000001;
			if(next_state == st_done)
			begin
				counter_9 = 4'b0000;
				counter_32 = 6'b000000;
			end
			if(next_state == st_go)
			begin
				counter_9 = 4'b0000;
			end
			current_state <= next_state;
		end
	end
	 

	
	assign read_1 = ((current_state == st_col_read) & (counter_9 == 4'b0001)) ? 1'b1: 1'b0;
	assign read_2 = ((current_state == st_col_read) & (counter_9 == 4'b0010)) ? 1'b1: 1'b0;
	assign read_3 = ((current_state == st_col_read) & (counter_9 == 4'b0011)) ? 1'b1: 1'b0;
	assign read_4 = ((current_state == st_col_read) & (counter_9 == 4'b0100)) ? 1'b1: 1'b0;
	assign read_5 = ((current_state == st_col_read) & (counter_9 == 4'b0101)) ? 1'b1: 1'b0;
	assign read_6 = ((current_state == st_col_read) & (counter_9 == 4'b0110)) ? 1'b1: 1'b0;
   	assign read_7 = ((current_state == st_col_read) & (counter_9 == 4'b0111)) ? 1'b1: 1'b0;
	assign read_8 = ((current_state == st_col_read) & (counter_9 == 4'b1000)) ? 1'b1: 1'b0;
	assign read_9 = ((current_state == st_col_read) & (counter_9 == 4'b1001)) ? 1'b1: 1'b0;

	
	
	assign In_FIFO_Read = ((current_state == st_col_read)) ? 1'b1 : 1'b0;
	assign Out_FIFO_Write = done;
	
	assign done = (current_state == st_done) ? 1'b1 : 1'b0;
	assign go = (current_state == st_go) ? 1'b1 : 1'b0;
	assign internal_reset = (current_state == st_reset) ? 1'b1 : 1'b0;
	
	
	nn_register_32 reg1(Clk,In_Data_FIFO,~Reset,read_1,out_reg1);
	nn_register_32 reg2(Clk,In_Data_FIFO,~Reset,read_2,out_reg2);
	nn_register_32 reg3(Clk,In_Data_FIFO,~Reset,read_3,out_reg3);
	nn_register_32 reg4(Clk,In_Data_FIFO,~Reset,read_4,out_reg4);
	nn_register_32 reg5(Clk,In_Data_FIFO,~Reset,read_5,out_reg5);
	nn_register_32 reg6(Clk,In_Data_FIFO,~Reset,read_6,out_reg6);
	nn_register_32 reg7(Clk,In_Data_FIFO,~Reset,read_7,out_reg7);
	nn_register_32 reg8(Clk,In_Data_FIFO,~Reset,read_8,out_reg8);
	nn_register_32 reg9(Clk,In_Data_FIFO,~Reset,read_9,out_reg9);

   	reduction_unit ru(Clk, internal_reset, go, out_reg1, out_reg2, out_reg3, out_reg4, out_reg5, out_reg6, out_reg7, out_reg8, out_reg9, internal_done, out[8], out[7], out[6], out[5], out[4], out[3], out[2], out[1], out[0]);
	assign Out_Data_FIFO[8:0] = out[8:0];
      
	assign Out_Data_FIFO[31:9] = 23'b00000000000000000000000;
	
	
	
endmodule
